@@ -1,33 +1,59 @@
 iwc <- 
-function(theta_R, theta_S, alpha, n, a, b, graph = TRUE,
-	xlab = "Matric potential (hPa)", ylab = "Water content", ...)
+function(theta_R, theta_S, alpha, n, a, b, hos = 0, 
+      graph = TRUE, xlab = "Matric potential (hPa)", 
+      ylab = "Water content", ...)
 {
+    stopifnot(hos >= 0)
+
     # Ch (van Genuchten's first derivative)
     x <- NULL
-    der <- function(x, theta_R, theta_S, alpha, n) {
-        out <- abs((theta_S - theta_R) * (1 + (alpha * x)^n)^(1/n - 
-           1) * (1/n - 1) * (alpha * x)^n * (n/(x * (1 + (alpha * 
-           x)^n))))
+    der. <- function(x, theta_R, theta_S, alpha, n) 
+    {
+        out <- abs((theta_S - theta_R) * 
+           (1 + (alpha * x)^n)^(1/n - 1) * 
+           (1/n - 1) * (alpha * x)^n * 
+           (n/(x * (1 + (alpha * x)^n))))
+        return(out)
+    }
+
+    # weight limiting function for salinity
+    ws <- function(x, theta_R, theta_S, alpha, n)
+    {
+        theta <- soilwater(x, theta_R, theta_S, alpha, n)
+        out <- ( 1 + hos * theta_S * theta^-2 *
+           der.(x, theta_R, theta_S, alpha, n) )^-1
+        return(out)
+    }
+
+    # Es
+    der <- function(x, theta_R, theta_S, alpha, n)
+    {
+        out <- ws(x, theta_R, theta_S, alpha, n) *
+           der.(x, theta_R, theta_S, alpha, n)
         return(out)
     }
 
     # Hydraulic conductivity limiting function
-    Kr <- function(x, theta_R, theta_S, alpha, n) {
-       out <- (1 - (alpha*x)^(n-1) * (1 + (alpha*x)^n)^(1/n - 1) )^2 *
+    Kr <- function(x, theta_R, theta_S, alpha, n) 
+    {
+       out <- (1 - (alpha*x)^(n-1) * 
+          (1 + (alpha*x)^n)^(1/n - 1) )^2 *
           (1 + (alpha*x)^n)^( (1 - n) / 2*n)
        return(out)
     }
 
     # Kr weight function
-    wK <- function(x, theta_R, theta_S, alpha, n) {
+    wK <- function(x, theta_R, theta_S, alpha, n) 
+    {
         out <- ( Kr(x = 330, theta_R, theta_S, alpha, n) / 
-            Kr(x, theta_R, theta_S, alpha, n) )^0.08
+           Kr(x, theta_R, theta_S, alpha, n) )^0.08
         return(out)
     }
 
-    EK <- function(x, theta_R, theta_S, alpha, n) {
+    EK <- function(x, theta_R, theta_S, alpha, n) 
+    {
         out <- wK(x, theta_R, theta_S, alpha, n) * 
-            der(x, theta_R, theta_S, alpha, n)
+           der(x, theta_R, theta_S, alpha, n)
         return(out)
     }
 
@@ -35,14 +61,16 @@ function(theta_R, theta_S, alpha, n, a, b, graph = TRUE,
     potential <- function(theta, theta_R, theta_S, alpha, n)
     {
         m <- 1 - 1/n
-        out <- (( (theta_S - theta_R) / (theta - theta_R) )^(1/m) - 1 )^(1/n) * 1/alpha
+        out <- (( (theta_S - theta_R) / 
+           (theta - theta_R) )^(1/m) - 1 )^(1/n) * 1/alpha
         return(out)
     }
     h10 <- potential(theta_S - 0.10, theta_R, theta_S, alpha, n)
     h15 <- potential(theta_S - 0.15, theta_R, theta_S, alpha, n)
 
     # weight function for air porosity limiting function
-    wa <- function(x) {
+    wa <- function(x) 
+    {
         A <- 1 / ( log10( h15 / h10 ) )
         out <- A * log10(x / h10 )
         for(i in 1:length(x)) {
@@ -52,14 +80,11 @@ function(theta_R, theta_S, alpha, n, a, b, graph = TRUE,
         return(out)
     }
 
-    Ea <- function(x, theta_R, theta_S, alpha, n) {
-        out <- wa(x) * der(x, theta_R, theta_S, alpha, n)
-        return(out)
-    }
-
-    EKa <- function(x, theta_R, theta_S, alpha, n) {
-        out <- wK(x, theta_R, theta_S, alpha, n) * wa(x) * 
-        der(x, theta_R, theta_S, alpha, n)
+    EKa <- function(x, theta_R, theta_S, alpha, n) 
+    {
+        out <- wK(x, theta_R, theta_S, alpha, n) * 
+           wa(x) * 
+           der(x, theta_R, theta_S, alpha, n)
         return(out)
     }
 
@@ -70,7 +95,8 @@ function(theta_R, theta_S, alpha, n, a, b, graph = TRUE,
         return(out)
     }
 
-    ER <- function(x, a, b, theta_R, theta_S, alpha, n) {
+    ER <- function(x, a, b, theta_R, theta_S, alpha, n) 
+    {
         out <- wR(x, a, b) *
            der(x, theta_R, theta_S, alpha, n)
         return(out)
@@ -81,43 +107,42 @@ function(theta_R, theta_S, alpha, n, a, b, graph = TRUE,
     h2.5 <- (2.5 / a) ^ (1/b)
 
     # dry hydraulic conductivity
-    Kdry <- Kr(x = seq(12000, 15000, by = 50), theta_R, theta_S, alpha, n)
-    fit2 <- fitsoilwater4(theta = Kdry, psi = seq(12000, 15000, by = 50), 
+    Kdry <- Kr(x = seq(12000, 15000, by = 50), 
+        theta_R, theta_S, alpha, n)
+    fit2 <- fitsoilwater4(theta = Kdry, 
+        psi = seq(12000, 15000, by = 50), 
         model = "Ross")
 
     # weight limiting function for Kdry
     wKdry <- function(x, d)
     {
         out <- (12000 / x) ^ (-d)
-       return(out)
-    }
-
-    EKdry <- function(x, d, theta_R, theta_S, alpha, n) 
-    {
-        out <- wKdry(x, d) *
-        der(x, theta_R, theta_S, alpha, n)
         return(out)
     }
 
     ERKdry <- function(x, a, b, d, theta_R, theta_S, alpha, n) 
     {
         out <- wR(x, a, b) * wKdry(x, d) *
-        der(x, theta_R, theta_S, alpha, n)
+           der(x, theta_R, theta_S, alpha, n)
         return(out)
     }
 
+    # ----------------------------------------------
     # IWC integrations
     # EKa
     ave1 <- trapez(x = seq(h10, h15, length = 1000), 
-        y = EKa(x = seq(h10, h15, length = 1000), theta_R, theta_S, alpha, n))
+        y = EKa(x = seq(h10, h15, length = 1000), 
+           theta_R, theta_S, alpha, n))
 
     # EK
     ave2 <- trapez(x = seq(h15, 330, length = 1000), 
-        y = EK(x = seq(h15, 330, length = 1000), theta_R, theta_S, alpha, n))
+        y = EK(x = seq(h15, 330, length = 1000), 
+           theta_R, theta_S, alpha, n))
 
     # Ch
     ave3 <- trapez(x = seq(330, h1.5, length = 1000), 
-        y = der(x = seq(330, h1.5, length = 1000), theta_R, theta_S, alpha, n))
+        y = der(x = seq(330, h1.5, length = 1000), 
+           theta_R, theta_S, alpha, n))
 
     # ER
     if (h2.5 > 15000) h2.5. <- 15000 else h2.5. <- h2.5
@@ -130,6 +155,7 @@ function(theta_R, theta_S, alpha, n, a, b, graph = TRUE,
         y = ERKdry(x = seq(12000, 15000, length = 1000), 
             a, b,	d = coef(fit2)[2], theta_R, theta_S, alpha, n))
 
+    # ------------------------------------------------
     # integral energy calculation
     # hEKa
     hEKa <- function(x, theta_R, theta_S, alpha, n) 
@@ -138,7 +164,8 @@ function(theta_R, theta_S, alpha, n, a, b, graph = TRUE,
         return(out)
     }
     ave1h <- trapez(x = seq(h10, h15, length = 1000), 
-        y = hEKa(x = seq(h10, h15, length = 1000), theta_R, theta_S, alpha, n))
+        y = hEKa(x = seq(h10, h15, length = 1000), 
+           theta_R, theta_S, alpha, n))
 
     # hEK
     hEK <- function(x, theta_R, theta_S, alpha, n) 
@@ -147,7 +174,8 @@ function(theta_R, theta_S, alpha, n, a, b, graph = TRUE,
         return(out)
     }
     ave2h <- trapez(x = seq(h15, 330, length = 1000), 
-        y = hEK(x = seq(h15, 330, length = 1000), theta_R, theta_S, alpha, n))
+        y = hEK(x = seq(h15, 330, length = 1000), 
+           theta_R, theta_S, alpha, n))
 
     # hCh
     hder <- function(x, theta_R, theta_S, alpha, n)
@@ -156,7 +184,8 @@ function(theta_R, theta_S, alpha, n, a, b, graph = TRUE,
         return(out)
     }
     ave3h <- trapez(x = seq(330, h1.5, length = 1000), 
-        y = hder(x = seq(330, h1.5, length = 1000), theta_R, theta_S, alpha, n))
+        y = hder(x = seq(330, h1.5, length = 1000), 
+           theta_R, theta_S, alpha, n))
 
     # hER
     hER <- function(x, a, b, theta_R, theta_S, alpha, n) 
@@ -179,30 +208,55 @@ function(theta_R, theta_S, alpha, n, a, b, graph = TRUE,
         y = hERKdry(x = seq(12000, 15000, length = 1000), 
             a, b,	d = coef(fit2)[2], theta_R, theta_S, alpha, n))
 
+    # -------------------------------------------------
     # graphic
     if (graph) {
         curve(der(x, theta_R, theta_S, alpha, n), 
             from = 0, to = 350, 
-            xlab = xlab, ylab = ylab, main = "Wet range", ...)
-        curve(EK(x, theta_R, theta_S, alpha, n), add = TRUE, lty = 3, col = 4)
-        curve(EKa(x, theta_R, theta_S, alpha, n), add = TRUE, lty = 2, col = 2)
-        legend("topright",  c(expression(C(h)), expression(E[R](h)), 
-             expression(E[RKdry](h))), lty = c(1, 3, 2), col = c(1, 4, 2), 
+            xlab = xlab, ylab = ylab, 
+            main = "Wet range", ...)
+        curve(EK(x, theta_R, theta_S, alpha, n), 
+           add = TRUE, lty = 3, col = 4)
+        curve(EKa(x, theta_R, theta_S, alpha, n), 
+           add = TRUE, lty = 2, col = 2)
+        legend("topright",  c(expression(C(h)), 
+             expression(E[R](h)), expression(E[RKdry](h))), 
+             lty = c(1, 3, 2), col = c(1, 4, 2), 
              bg = "lightyellow")
     
         dev.new(width = 5, height = 5)
         curve(der(x, theta_R, theta_S, alpha, n), 
             from = h1.5, to = 15000, 
-            xlab = xlab, ylab = ylab, main = "Dry range", ...)
+            xlab = xlab, ylab = ylab, 
+            main = "Dry range", ...)
         curve(ER(x, a, b, theta_R, theta_S, alpha, n), 
-            add = TRUE, from = h1.5, to = h2.5., lty = 3, col = 4)
-        curve(ERKdry(x, a, b, d = coef(fit2)[2], theta_R, theta_S, alpha, n), 
-            add = TRUE, from = 12000, to = 15000, lty = 2, col = 2)
-        legend("topright",  c(expression(C(h)), expression(E[R](h)), 
-             expression(E[RKdry](h))), lty = c(1, 3, 2), col = c(1, 4, 2), 
+            add = TRUE, from = h1.5, to = h2.5., 
+            lty = 3, col = 4)
+        curve(ERKdry(x, a, b, d = coef(fit2)[2], 
+            theta_R, theta_S, alpha, n), 
+            add = TRUE, from = 12000, to = 15000, 
+            lty = 2, col = 2)
+        legend("topright",  c(expression(C(h)), 
+             expression(E[R](h)), expression(E[RKdry](h))), 
+             lty = c(1, 3, 2), col = c(1, 4, 2), 
              bg = "lightyellow")
+
+        if (hos != 0) {
+           dev.new(width = 5, height = 5)
+           curve(der.(x, theta_R, theta_S, alpha, n), 
+              from = 1, to = 15000, log = "x",
+              xlab = xlab, ylab = ylab, 
+              main = "Salinity effect", ...)
+           curve(der(x, theta_R, theta_S, alpha, n), 
+               add = TRUE, from = 1, to = 15000, 
+               log = "x", col = 2, lty = 2)
+           legend("topright", c(expression(C(h)), 
+               expression(E[S](h))), lty = c(1, 2), 
+               col = c(1, 2), bg = "lightyellow")
+        }        
     }
 
+    # ----------------------------------------
     # output
     IWC <- ave1 + ave2 + ave3 + ave4 + ave5
     EI <- (ave1h + ave2h + ave3h + ave4h + ave5h) / (10 * IWC)
@@ -210,7 +264,7 @@ function(theta_R, theta_S, alpha, n, a, b, graph = TRUE,
 }
 
 
-# ---------------------------------------------------
+# --------------------------------------------
 # numerical integration: trapezoidal rule
 trapez <- 
 function(x, y)
